from bs4 import BeautifulSoup as bs4
from lyricli.exceptions import ParserException

class Parser:
    def feed(self, html):
        if(html.status_code == 200):
            html = html.text
            soup = bs4(html, "lxml")
            try:
                songId = soup.find("div", attrs={"class":"section_song"}).find_all("td")[0].find("input")['value']
            except AttributeError:
                raise ParserException(808)
                
        else:
            raise ParserException(html.status_code)

        return songId

    def feed_tab(self, html):
        if(html.status_code == 200):
            html = html.text
            time_table = {}
            soup = bs4(html, "lxml")
            lyrics = soup.find("div", attrs={"class": "lyric"})
            if lyrics:
                for c, i in enumerate(lyrics.strings):
                    time_table[i.strip()] = c
            else:
                raise ParserException(800)
        else:
            ParserException("Status Code", html.status_code)

        return time_table


# -*- coding: utf-8 -*-

import sys

from argparse import ArgumentParser, ArgumentTypeError
from collections import namedtuple

from lyricli.exceptions import ConsoleException, ParserException
from lyricli.melon import get_lyrics
from lyricli.output import print_out, print_err
from lyricli.__init__ import __version__

Command = namedtuple("Command", ["name", "description", "arguments"])
CLIENT_WEBSITE = ""

COMMANDS = [
    Command(
        name="lyrics",
        description="가사 정보를 보여줍니다.",
        arguments=[
            (["-t", "--title"], {
                "help": "노래 제목",
                "type": str,
            }),
            (["-r", "--artist"], {
                "help": "가수 이름",
                "type": str,
            })
        ]
    )
]

COMMON_ARGUMENTS = [
    (["--debug"], {
        "help": "show debug log in console",
        "action": 'store_true',
        "default": False,
    }),
    (["--no-color"], {
        "help": "disable ANSI colors in output",
        "action": 'store_true',
        "default": False,
    })
]


def get_parser():
    description = "멜론에서 노래 가사를 가져오는 간단한 스크립트"
    parser = ArgumentParser(
        prog="lyricli", description=description, epilog=CLIENT_WEBSITE)
    parser.add_argument(
        "--version", help="현재 버전 정보를 알려줍니다.", action="store_true")

    subparsers = parser.add_subparsers(title="commands")

    for command in COMMANDS:
        sub = subparsers.add_parser(command.name, help=command.description)
        
        sub.set_defaults(func=get_lyrics)

        for args, kwargs in command.arguments + COMMON_ARGUMENTS:
            sub.add_argument(*args, **kwargs)

    return parser


def main():
    parser = get_parser()
    args = parser.parse_args()

    if args.version:
        print("lyricli v{}".format(__version__))
        return

    if "func" not in args:
        parser.print_help()
        return

    try:
        args.func(args)
    except ConsoleException as e:
        print_err(e)
        sys.exit(1)
    except KeyboardInterrupt:
        print_err("Operation canceled")
        sys.exit(1)
    except ParserException as e:
        print_err(e)
        sys.exit(1)

if __name__ == '__main__':
    main()


"""
    멜론 API에 접속합니다.
"""

import requests

from lyricli.exceptions import ConsoleException, ParserException
from lyricli.output import print_out
from lyricli.parser import Parser

BASE_URL = "https://www.melon.com/"
INDEX_POSTFIX = "search/total/index.htm?q="
DETAIL_POSTFIX = "song/detail.htm?songId="

def get_lyrics(args):
    title = "+".join(args.title.split())
    artist = args.artist if args.artist else ""
    artist = "+".join(artist.split())
    query_string = "{}+{}".format(title, artist)
    url = '{}{}{}'.format(BASE_URL, INDEX_POSTFIX, query_string)
    headers = {
        'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_2) AppleWebKit/601.3.9 (KHTML, like Gecko) Version/9.0.2 Safari/601.3.9'
        }
    html = requests.get(url, headers=headers)

    parser = Parser()
    songId = parser.feed(html)
    tab_url = '{}{}{}'.format(BASE_URL, DETAIL_POSTFIX, songId)
    html_tab = requests.get(tab_url, headers=headers)
    lyric_table = parser.feed_tab(html_tab)
    for i in lyric_table:
        print_out("<blue>{}</blue>".format(i))
    
    return lyric_table




class ConsoleException(Exception):
    """Raised when an error occurs and script execution should halt"""
    pass

class ParserException(Exception):
    """Raised when an error occurs during parsing Melon HTML"""
    def __init__(self, error_code: int):
        if(error_code == 808):
            msg = "해당 쿼리로 멜론에서 곡을 찾을 수 없습니다. 다른 키워드로 입력하세요."
        elif(error_code == 800):
            msg = "멜론에 가사 정보가 없습니다."
        else:
            msg = "인터넷 연결을 확인하세요."
        self.message = "{} - {}".format(error_code, msg)
        super().__init__(self.message)


#!/usr/bin/env python

from setuptools import setup, find_packages

long_description = """
Quickly find lyrics from your terminal. 
Uses Melon service.
"""
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

requires = [
    "beautifulsoup4>=4.10.0",
    "six>=1.16.0",
    "lxml>=4.6.3",
    "requests>=2.26.0",
]

setup(
    name="lyricli",
    version="0.0.9",
    author="Sean Hong",
    author_email="seanhong2000@gmail.com",
    description="Cli tool made with python for viewing lyrics",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/KKodiac/lyricli",
    project_urls={
        "Bug Tracker": "https://github.com/KKodaic/lyricli/issues",
    },
    classifiers=[
        "Environment :: Console",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3 :: Only",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=find_packages(),
    python_requires=">=3.6",
    entry_points={
        'console_scripts': [
            'lyricli = lyricli.console:main',
        ],
    }
)